//============================================================================
// TreeEval.g - Parse, Create an AST, and Evaluate
//
// Author:  Bary W Pollack
// Date:    Mar. 06, 2004 - 17:00
//
// This example shows how to create an AST and then walk it
// in order to create the corresponding semantics (evaluation)
//
//============================================================================

// Import the necessary classes
{
import java.io.*;
import java.util.*;
}


//----------------------------------------------------------------------------
// Define a Parser, call it TreeEvalParser
//----------------------------------------------------------------------------

class TreeEvalParser extends Parser;

options {
    buildAST = true;
}

{
    public static void main(String[] args) throws Exception {
        TreeEvalLexer  lexer  = new TreeEvalLexer(System.in);
        TreeEvalParser parser = new TreeEvalParser(lexer);
        parser.expr();

        // Create an AST for the input expression
        AST tree = parser.getAST();
        System.out.println("Preorder traversal: " + tree.toStringTree());

        // Walk the AST to create the semantics (evaluate the expression)
        TreeEvalWalker treeWalker = new TreeEvalWalker();
        int nValue = treeWalker.expr(tree);

        System.out.println();
        System.out.println("* Success; nValue = " + nValue + " *");
        System.out.println();
    }
}


//==================================================================
// TreeEval Syntax - simplified arithmetic expressions
//==================================================================

expr
    :  mexpr ((PLUS^|MINUS^) mexpr)*
    ;

mexpr
    :  atom (STAR^ atom)*
    ;

atom
    :  INT
    |  LPAREN! expr RPAREN!
    ;


//----------------------------------------------------------------------------
// The TreeEval Scanner
//----------------------------------------------------------------------------

class TreeEvalLexer extends Lexer;

options {
    charVocabulary = '\0'..'\377';
    k = 2;
}

LPAREN : '(' ;
RPAREN : ')' ;
PLUS   : '+' ;
MINUS  : '-' ;
STAR   : '*' ;

INT    : ('0'..'9')+ ;

WS     : ( ' '
         | '\r' '\n'
         | '\n'
         | '\t'
         )
         {$setType(Token.SKIP);}
       ;    


//----------------------------------------------------------------------------
// The TreeEval Walker
//----------------------------------------------------------------------------

class TreeEvalWalker extends TreeParser;

options {
    importVocab = TreeEvalParser;
}

expr returns [int r = 0]
{ int a, b; }
    :  #(PLUS  a=expr b=expr)  { r = a + b; }
    |  #(MINUS a=expr b=expr)  { r = a - b; }   
    |  #(STAR  a=expr b=expr)  { r = a * b; }
    |  i:INT                   { r = (int)Integer.parseInt(i.getText()); }
    ;


// EOF
